"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = exports.bollingerBandsArray = void 0;

var _binaryUtils = require("binary-utils");

var _math = require("./math");

var _simpleMovingAverage = _interopRequireDefault(require("./simpleMovingAverage"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var bollingerBands = function bollingerBands(data, config) {
  var _config$periods = config.periods,
      periods = _config$periods === void 0 ? 20 : _config$periods,
      field = config.field,
      _config$stdDevUp = config.stdDevUp,
      stdDevUp = _config$stdDevUp === void 0 ? 2 : _config$stdDevUp,
      _config$stdDevDown = config.stdDevDown,
      stdDevDown = _config$stdDevDown === void 0 ? 2 : _config$stdDevDown,
      _config$pipSize = config.pipSize,
      pipSize = _config$pipSize === void 0 ? 2 : _config$pipSize;
  var vals = (0, _math.takeLast)(data, periods, field);
  var middle = (0, _simpleMovingAverage["default"])(vals, {
    periods: periods
  });
  var stdDev = (0, _math.stddev)(vals);
  var upper = middle + stdDev * stdDevUp;
  var lower = middle - stdDev * stdDevDown;
  return [+middle.toFixed(pipSize), +upper.toFixed(pipSize), +lower.toFixed(pipSize)];
};

var bollingerBandsArray = function bollingerBandsArray(data, config) {
  var periods = config.periods;
  return (0, _binaryUtils.sequence)(data.length - periods + 1).map(function (x, i) {
    return bollingerBands(data.slice(i, i + periods), config);
  });
};

exports.bollingerBandsArray = bollingerBandsArray;
var _default = bollingerBands;
exports["default"] = _default;